@@ -29,7 +29,7 @@ NULL
 #' @param cv A boolean argument, indicates whether the user will apply cross validation to select tuning parameter, default is FALSE
 #' @param nfold An positive integer, the number of folds for cross validation
 #' @param verbose If is TRUE, then it will print all information about current step.
-#' @return A list object including 
+#' @return A list object including
 #' \describe{
 #'     \item{data}{the original dataset}
 #'     \item{q}{the time lag for the time series, in this case, it is 1}
@@ -49,62 +49,63 @@ NULL
 #' signals <- c(-0.7, 0.8)
 #' singular_vals <- c(1, 0.75, 0.5)
 #' info_ratio <- rep(0.35, 2)
-#' try <- simu_var(method = "LS", nob = nob, k = p, lags = 1, brk = brk, 
-#'                 sigma = as.matrix(diag(p)), signals = signals, 
-#'                 rank = rank, singular_vals = singular_vals, info_ratio = info_ratio, 
+#' try <- simu_var(method = "LS", nob = nob, k = p, lags = 1, brk = brk,
+#'                 sigma = as.matrix(diag(p)), signals = signals,
+#'                 rank = rank, singular_vals = singular_vals, info_ratio = info_ratio,
 #'                 sp_pattern = "off-diagonal", spectral_radius = 0.9)
 #' data <- try$series
-#' 
+#'
 #' lambda1 = lambda2 = lambda3 <- c(2.5, 2.5)
 #' mu1 = mu2 = mu3 <- c(15, 15)
-#' fit <- lstsp(data, lambda.1 = lambda1, mu.1 = mu1, 
-#'              lambda.2 = lambda2, mu.2 = mu2, 
-#'              lambda.3 = lambda3, mu.3 = mu3, alpha_L = 0.25, 
-#'              step.size = 5, niter = 20, skip = 5, 
+#' fit <- lstsp(data, lambda.1 = lambda1, mu.1 = mu1,
+#'              lambda.2 = lambda2, mu.2 = mu2,
+#'              lambda.3 = lambda3, mu.3 = mu3, alpha_L = 0.25,
+#'              step.size = 5, niter = 20, skip = 5,
 #'              cv = FALSE, verbose = FALSE)
 #' summary(fit)
 #' plot(fit, data, display = "cp")
 #' plot(fit, data, display = "param")
 #' }
-lstsp <- function(data, lambda.1 = NULL, mu.1 = NULL, lambda.1.seq = NULL, mu.1.seq = NULL, 
-                  lambda.2, mu.2, lambda.3, mu.3, alpha_L = 0.25, omega = NULL, h = NULL, 
-                  step.size = NULL, tol = 1e-4, niter = 100, backtracking = TRUE, skip = 5, 
+lstsp <- function(data, lambda.1 = NULL, mu.1 = NULL, lambda.1.seq = NULL, mu.1.seq = NULL,
+                  lambda.2 = NULL, mu.2 = NULL, lambda.3 = NULL, mu.3 = NULL, alpha_L = 0.25, omega = NULL, h = NULL,
+                  step.size = NULL, tol = 1e-4, niter = 100, backtracking = TRUE, skip = 5,
                   cv = FALSE, nfold = NULL, verbose = FALSE){
+
     ###### model dimensions
     n <- dim(data)[1]
     p <- dim(data)[2]
     if(is.null(h)){
-        h <- 2*floor(sqrt(n))+1
+        h <- 2 * floor(sqrt(n)) + 1
     }
-    
+
     ###### Step 1: rolling-window find candidate change points ######
     start.time <- proc.time()[3]
-    candi_pts <- first.step.detect(data = data, h = h, step.size = step.size, 
-                                   lambda = lambda.1, mu = mu.1, alpha_L = alpha_L, 
+    candi_pts <- first.step.detect(data = data, h = h, step.size = step.size,
+                                   lambda = lambda.1, mu = mu.1, alpha_L = alpha_L,
                                    skip = skip, lambda.1.seq = lambda.1.seq, mu.1.seq = mu.1.seq,
                                    cv = cv, nfold = nfold, verbose = verbose)
-    
+
     ###### Step 2: screening out redundant candidate and obtain the final change points ######
     N <- n - 1
     if(is.null(omega)){
-        omega <- (1/0.05) * (((log(N))^1)*log(p))
+        omega <- (1 / 0.05) * log(N) * log(p)
     }
-    screen_select <- second.step.detect(data, candi_pts, omega, lambda.2, mu.2, 
+    screen_select <- second.step.detect(data, candi_pts, omega, lambda.2, mu.2,
                                         alpha_L = alpha_L, verbose = verbose)
     final.pts <- screen_select$pts
-    
+
     ###### Step 3: estimate model parameters by using final change points ######
     pts <- c(1, final.pts, n)
-    est_phi = sp_mats = lr_mats <- vector('list', length(pts)-1)
-    for(i in 1:(length(pts)-1)){
+    est_phi = sp_mats = lr_mats <- vector('list', length(pts) - 1)
+    for(i in 1:(length(pts) - 1)){
         s <- pts[i]
         e <- pts[i+1]
         interval_data <- data[s:e, ]
         ns <- dim(interval_data)[1]
-        
+
         X <- interval_data[1:(ns-1), ]
         y <- interval_data[2:ns, ]
-        fit <- fista.LpS(X, y, lambda = lambda.3, mu = mu.3, alpha_L = alpha_L, 
+        fit <- fista.LpS(X, y, lambda = lambda.3, mu = mu.3, alpha_L = alpha_L,
                          niter = niter, backtracking, x.true = diag(p))
         sp_mats[[i]] <- t(fit$sparse.comp)
         lr_mats[[i]] <- t(fit$lr.comp)
@@ -112,14 +113,14 @@ lstsp <- function(data, lambda.1 = NULL, mu.1 = NULL, lambda.1.seq = NULL, mu.1.
     }
     end.time <- proc.time()[3]
     time.comparison <- end.time - start.time
-    
+
     # S3 class result
-    final.result <- structure(list(data = data, 
-                                   q = 1, 
-                                   cp = final.pts, 
-                                   sparse_mats = sp_mats, 
-                                   lowrank_mats = lr_mats, 
-                                   est_phi = est_phi, 
+    final.result <- structure(list(data = data,
+                                   q = 1,
+                                   cp = final.pts,
+                                   sparse_mats = sp_mats,
+                                   lowrank_mats = lr_mats,
+                                   est_phi = est_phi,
                                    time = time.comparison), class = "VARDetect.result")
     return(final.result)
 }
@@ -132,8 +133,8 @@ lstsp <- function(data, lambda.1 = NULL, mu.1 = NULL, lambda.1.seq = NULL, mu.1.
 #' @param data A n by p dataset, n is the number of observations, p is the number of variables
 #' @param lambda A 2-d vector recording tuning parameters for sparse components left/right
 #' @param mu A 2-d vector recording tuning parameters for low rank components left/right
-#' @param alpha_L A numeric value, a positive number indicating the constraint space for low rank matrix 
-#' @param skip The number of observations at the boundaries we should skip 
+#' @param alpha_L A numeric value, a positive number indicating the constraint space for low rank matrix
+#' @param skip The number of observations at the boundaries we should skip
 #' @return A list object, which includes the followings:
 #' \describe{
 #'   \item{cp}{A vector recording all estimated change points}
@@ -144,47 +145,47 @@ lstsp <- function(data, lambda.1 = NULL, mu.1 = NULL, lambda.1.seq = NULL, mu.1.
 #'   \item{sse}{The sum of squared errors for all passed time points}
 #' }
 #' @keywords internal
-#' 
+#'
 detect.LpS <- function(data, lambda, mu, alpha_L, skip = 50){
     sse <- c()
     n <- dim(data)[1]
     k <- dim(data)[2]
-    
-    lr_est <- vector("list", n - 2*skip)
-    sp_est <- vector('list', n - 2*skip)
+
+    lr_est <- vector("list", n - 2 * skip)
+    sp_est <- vector('list', n - 2 * skip)
     for(t in (skip+1):(n-skip)){
         ###### segmentation of dataset ######
         seg1 <- data[1:t,]
-        seg2 <- data[((t+1):n),]
-        
+        seg2 <- data[(t + 1):n,]
+
         ### fitting the left-side dataset
-        X1 <- seg1[1:(t-1),]
+        X1 <- seg1[1:(t - 1),]
         Y1 <- seg1[2:t,]
-        fit1 <- fista.LpS(X1, Y1, lambda = lambda[1], mu = mu[1], alpha_L = alpha_L, 
+        fit1 <- fista.LpS(X1, Y1, lambda = lambda[1], mu = mu[1], alpha_L = alpha_L,
                           niter = 100, backtracking = TRUE, x.true = diag(k))
-        
+
         ### fitting the right-side dataset
-        X2 <- seg2[1:(n-t-1),]
-        Y2 <- seg2[2:(n-t),]
-        fit2 <- fista.LpS(X2, Y2, lambda = lambda[2], mu = mu[2], alpha_L = alpha_L, 
+        X2 <- seg2[1:(n - t - 1),]
+        Y2 <- seg2[2:(n - t),]
+        fit2 <- fista.LpS(X2, Y2, lambda = lambda[2], mu = mu[2], alpha_L = alpha_L,
                           niter = 100, backtracking = TRUE, x.true = diag(k))
-        
+
         ###### find the residual and full SSE ######
         phi.hat1 <- t(fit1$lr.comp + fit1$sparse.comp)
         phi.hat2 <- t(fit2$lr.comp + fit2$sparse.comp)
-        
+
         lr_est[[t-skip]] <- cbind(t(fit1$lr.comp), t(fit2$lr.comp))
         sp_est[[t-skip]] <- cbind(t(fit1$sparse.comp), t(fit2$sparse.comp))
-        
+
         pred.err1 <- (1/t) * norm(Y1 - X1 %*% t(phi.hat1), "F")^2
         pred.err2 <- (1/(n-t)) * norm(Y2 - X2 %*% t(phi.hat2), "F")^2
         sse <- c(sse, pred.err1 + pred.err2)
     }
-    
+
     idx <- which.min(sse) + skip
-    
-    L_hat1 <- lr_est[[idx-skip]][,c(1:k)]; L_hat2 <- lr_est[[idx-skip]][,-(c(1:k))]
-    S_hat1 <- sp_est[[idx-skip]][,c(1:k)]; S_hat2 <- sp_est[[idx-skip]][,-(c(1:k))]
+
+    L_hat1 <- lr_est[[idx - skip]][,c(1:k)]; L_hat2 <- lr_est[[idx - skip]][,-(c(1:k))]
+    S_hat1 <- sp_est[[idx - skip]][,c(1:k)]; S_hat2 <- sp_est[[idx - skip]][,-(c(1:k))]
     return(list(cp = idx, S_hat1 = S_hat1, S_hat2 = S_hat2, L_hat1 = L_hat1, L_hat2 = L_hat2, sse = sse))
 }
 
@@ -208,66 +209,66 @@ detect.LpS <- function(data, lambda, mu, alpha_L, skip = 50){
 #'   \item{sse}{The sum of squared errors for all passed time points}
 #' }
 #' @keywords internal
-#' 
-cv.detect.LpS <- function(data, lambda.1.seq = NULL, mu.1.seq = NULL, 
+#'
+cv.detect.LpS <- function(data, lambda.1.seq = NULL, mu.1.seq = NULL,
                           alpha_L = 0.25, skip = 50, nfold = 5){
     sse <- c()
     n <- dim(data)[1]
     k <- dim(data)[2]
 
     if(is.null(lambda.1.seq)){
-        lambda.1.seq <- sqrt(log(k)/n) * exp(seq(-2, 5, length.out = nfold))
+        lambda.1.seq <- sqrt(log(k) / n) * exp(seq(-2, 5, length.out = nfold))
     }
     if(is.null(mu.1.seq)){
-        mu.1.seq <- sqrt((k)/n) * exp(seq(-2, 5, length.out = nfold))
+        mu.1.seq <- sqrt((k) / n) * exp(seq(-2, 5, length.out = nfold))
     }
-    
-    lr_est <- vector("list", n-2*skip)
-    sp_est <- vector('list', n-2*skip)
-    for(t in (skip+1):(n-skip)){
+
+    lr_est <- vector("list", n - 2 * skip)
+    sp_est <- vector('list', n - 2 * skip)
+    for(t in (skip + 1):(n - skip)){
         ###### segmentation of dataset ######
         seg1 <- data[1:t,]
-        seg2 <- data[((t+1):n),]
-        
+        seg2 <- data[(t + 1):n,]
+
         ###### tuning parameter selection for the left-side segment ######
-        ret <- cv.tuning.selection(seg1, lambda.1.seq, mu.1.seq, 
+        ret <- cv.tuning.selection(seg1, lambda.1.seq, mu.1.seq,
                                    alpha_L = alpha_L, nfold = nfold)
         lambda1 <- ret$lambda
         mu1 <- ret$mu
-        
+
         ### fitting the left-side dataset
-        X1 <- seg1[1:(t-1),]
+        X1 <- seg1[1:(t - 1),]
         Y1 <- seg1[2:t,]
-        fit1 <- fista.LpS(X1, Y1, lambda = lambda1, mu = mu1, alpha_L = alpha_L, 
+        fit1 <- fista.LpS(X1, Y1, lambda = lambda1, mu = mu1, alpha_L = alpha_L,
                           niter = 100, backtracking = TRUE, x.true = diag(k))
-        
+
         ###### tuning parameter selection for the right-side segment ######
-        ret <- cv.tuning.selection(seg2, lambda.1.seq, mu.1.seq, 
+        ret <- cv.tuning.selection(seg2, lambda.1.seq, mu.1.seq,
                                    alpha_L = alpha_L, nfold = nfold)
         lambda2 <- ret$lambda
         mu2 <- ret$mu
-        
+
         # fitting the right-side dataset
-        X2 <- seg2[1:(n-t-1),]
-        Y2 <- seg2[2:(n-t),]
-        fit2 <- fista.LpS(X2, Y2, lambda = lambda2, mu = mu2, alpha_L = alpha_L, 
+        X2 <- seg2[1:(n - t - 1),]
+        Y2 <- seg2[2:(n - t),]
+        fit2 <- fista.LpS(X2, Y2, lambda = lambda2, mu = mu2, alpha_L = alpha_L,
                           niter = 100, backtracking = TRUE, x.true = diag(k))
-        
+
         ###### find the residual and full SSE ######
         phi.hat1 <- t(fit1$lr.comp + fit1$sparse.comp)
         phi.hat2 <- t(fit2$lr.comp + fit2$sparse.comp)
-        
+
         lr_est[[t-skip]] <- cbind(t(fit1$lr.comp), t(fit2$lr.comp))
         sp_est[[t-skip]] <- cbind(t(fit1$sparse.comp), t(fit2$sparse.comp))
-        
-        pred.err1 <- (1/n) * norm(Y1 - X1 %*% t(phi.hat1), "F")^2
-        pred.err2 <- (1/n) * norm(Y2 - X2 %*% t(phi.hat2), "F")^2
+
+        pred.err1 <- (1 / n) * norm(Y1 - X1 %*% t(phi.hat1), "F")^2
+        pred.err2 <- (1 / n) * norm(Y2 - X2 %*% t(phi.hat2), "F")^2
         sse <- c(sse, pred.err1 + pred.err2)
     }
-    
+
     idx <- which.min(sse) + skip
-    L_hat1 <- lr_est[[idx-skip]][,c(1:k)]; L_hat2 <- lr_est[[idx-skip]][,-(c(1:k))]
-    S_hat1 <- sp_est[[idx-skip]][,c(1:k)]; S_hat2 <- sp_est[[idx-skip]][,-(c(1:k))]
+    L_hat1 <- lr_est[[idx - skip]][,c(1:k)]; L_hat2 <- lr_est[[idx - skip]][,-(c(1:k))]
+    S_hat1 <- sp_est[[idx - skip]][,c(1:k)]; S_hat2 <- sp_est[[idx - skip]][,-(c(1:k))]
     return(list(cp = idx, S_hat1 = S_hat1, S_hat2 = S_hat2, L_hat1 = L_hat1, L_hat2 = L_hat2, sse = sse))
 }
 
@@ -285,17 +286,15 @@ cv.detect.LpS <- function(data, lambda.1.seq = NULL, mu.1.seq = NULL,
 #'     \item{test}{test dataset}
 #' }
 #' @keywords internal
-#' 
+#'
 cv.separate <- function(period, nfold = 5){
     n <- dim(period)[1]
     if(n < 10){
-        # print("Warning: too few data points!")
-        # break
         stop("too few data points!")
     }else{
         cv.index <- c()
         for(i in 1:n){
-            if(i %% nfold == (nfold-1)){
+            if(i %% nfold == (nfold - 1)){
                 cv.index <- c(cv.index, i)
             }
         }
@@ -319,22 +318,22 @@ cv.separate <- function(period, nfold = 5){
 #'     \item{mu}{final selected tuning parameter for low rank}
 #' }
 #' @keywords internal
-#' 
+#'
 cv.tuning.selection <- function(data, lambda.seq, mu.seq, alpha_L = 0.25, nfold = 5){
     ### separate data time series into training and testing set
     separate <- cv.separate(data, nfold = nfold)
     data_train <- separate$train
     data_test <- separate$test
-    
+
     ntrain <- dim(data_train)[1]
     ntest <- dim(data_test)[1]
-    
-    X_train <- data_train[1:(ntrain-1),]
+
+    X_train <- data_train[1:(ntrain - 1),]
     Y_train <- data_train[2:ntrain,]
-    
-    X_test <- data_test[1:(ntest-1),]
+
+    X_test <- data_test[1:(ntest - 1),]
     Y_test <- data_test[2:ntest,]
-    
+
     ### grid search part: find the tuning parameter that minimizes the prediction error on test set
     n <- dim(data)[1]
     k <- dim(data)[2]
@@ -343,9 +342,9 @@ cv.tuning.selection <- function(data, lambda.seq, mu.seq, alpha_L = 0.25, nfold
         for(c in 1:length(mu.seq)){
             lambda <- lambda.seq[r]
             mu <- mu.seq[c]
-            fit <- fista.LpS(X_train, Y_train, lambda = lambda, mu = mu, alpha_L = alpha_L, 
+            fit <- fista.LpS(X_train, Y_train, lambda = lambda, mu = mu, alpha_L = alpha_L,
                              niter = 100, backtracking = TRUE, diag(k))
-            if(qr(fit$lr.comp)$rank == 0 | qr(fit$lr.comp)$rank > k/2){
+            if(qr(fit$lr.comp)$rank == 0 | qr(fit$lr.comp)$rank > k / 2){
                 grid[r,c] <- Inf
             }else{
                 residual <- Y_test - X_test %*% (fit$sparse.comp + fit$lr.comp)
@@ -373,7 +372,7 @@ cv.tuning.selection <- function(data, lambda.seq, mu.seq, alpha_L = 0.25, nfold
 #'  \item{L.n}{Value of objective function}
 #' }
 #' @keywords internal
-#' 
+#'
 break.var.lps <- function(data, pts, lambda, mu, alpha_L = 0.25){
     n <- dim(data)[1]
     k <- dim(data)[2]
@@ -389,9 +388,9 @@ break.var.lps <- function(data, pts, lambda, mu, alpha_L = 0.25){
         n.temp <- dim(data.temp)[1]
         x.temp <- data.temp[1:(n.temp-1),]
         y.temp <- data.temp[2:n.temp,]
-        
+
         ### estimate the coefficient matrices and calculate the corresponding prediction error
-        try <- fista.LpS(x.temp, y.temp, lambda = lambda, mu = mu, alpha_L = alpha_L, 
+        try <- fista.LpS(x.temp, y.temp, lambda = lambda, mu = mu, alpha_L = alpha_L,
                          niter = 20, backtracking = TRUE, diag(k))
         est.coef <- t(try$lr.comp) + t(try$sparse.comp)
         pred.error <- y.temp - x.temp %*% t(est.coef)
@@ -414,14 +413,14 @@ break.var.lps <- function(data, pts, lambda, mu, alpha_L = 0.25){
 #'     \item{L.n.current}{Current value of objective function}
 #' }
 #' @keywords internal
-#' 
+#'
 backward.selection <- function(data, pts, lambda, mu, alpha_L = 0.25){
     n <- dim(data)[1]
     k <- dim(data)[2]
     m <- length(pts)
     L.n <- rep(0, m)
     L.n.current <- rep(0, 1)
-    
+
     try <- break.var.lps(data, pts, lambda, mu)
     L.n.current <- try$L.n
     for(mm in 1:m){
@@ -454,13 +453,13 @@ backward.selection <- function(data, pts, lambda, mu, alpha_L = 0.25){
 #' @param verbose if TRUE, then all information for current stage are printed
 #' @return A vector which includes all candidate change points selected by rolling window
 #' @keywords internal
-#' 
-first.step.detect <- function(data, h, step.size = NULL, lambda, mu, alpha_L = 0.25, 
+#'
+first.step.detect <- function(data, h, step.size = NULL, lambda, mu, alpha_L = 0.25,
                               skip = 3, lambda.1.seq = NULL, mu.1.seq = NULL,
                               cv = FALSE, nfold = NULL, verbose = FALSE){
     n <- dim(data)[1]
     k <- dim(data)[2]
-    
+
     if(is.null(step.size)){
         step.size <- floor(0.25 * h)
     }
@@ -469,14 +468,14 @@ first.step.detect <- function(data, h, step.size = NULL, lambda, mu, alpha_L = 0
     candi_cp <- c()
     while(e <= n-1){
         interval_data <- data[s:e, ]
-        
+
         ### use single change point detection method to find the candidates
         if(!cv){
             fit <- detect.LpS(interval_data, lambda = lambda, mu = mu, alpha_L = alpha_L, skip = skip)
             current_cp <- fit$cp
             candi_cp <- c(candi_cp, current_cp + s)
         }else{
-            fit <- cv.detect.LpS(interval_data, lambda.1.seq = lambda.1.seq, mu.1.seq = mu.1.seq, 
+            fit <- cv.detect.LpS(interval_data, lambda.1.seq = lambda.1.seq, mu.1.seq = mu.1.seq,
                                  alpha_L = alpha_L, skip = skip, nfold = nfold)
             current_cp <- fit$cp
         }
@@ -487,12 +486,13 @@ first.step.detect <- function(data, h, step.size = NULL, lambda, mu, alpha_L = 0
             e <- n
         }
         if(verbose){
-            print(paste("Finished interval:", s, e))
+            cat(paste("Finished interval:", s, e))
+            cat("\n")
         }
     }
     first.step.cp <- candi_cp[-1]
     first.step.cp <- first.step.cp[order(first.step.cp)]
-    
+
     ### remove repeat candidate change points
     m <- length(first.step.cp)
     pts <- c()
@@ -523,7 +523,7 @@ first.step.detect <- function(data, h, step.size = NULL, lambda, mu, alpha_L = 0
 #'     \item{ic}{Values of information criterion}
 #' }
 #' @keywords internal
-#' 
+#'
 second.step.detect <- function(data, pts, omega, lambda, mu, alpha_L = 0.25, verbose = FALSE){
     m <- length(pts)
     if(m == 0){
@@ -540,7 +540,8 @@ second.step.detect <- function(data, pts, omega, lambda, mu, alpha_L = 0.25, ver
         }else{
             pts <- pts[-which(L.n == min(L.n))]
             if(verbose){
-                print(pts)
+                cat(pts)
+                cat("\n")
             }
         }
     }
@@ -558,7 +559,7 @@ second.step.detect <- function(data, pts, omega, lambda, mu, alpha_L = 0.25, ver
 #' @param lambda A positive numeric value, indicating the tuning parameter for sparse component
 #' @param mu A positive numeric value, indicating the tuning parameter for low rank component
 #' @param alpha_L The constraint coefficient of low rank component, default is 0.25
-#' @param niter The maximum number of iterations required for FISTA 
+#' @param niter The maximum number of iterations required for FISTA
 #' @param backtracking A boolean argument, indicating that use backtracking in the FISTA
 #' @param x.true A p by p matrix, the true model parameter. Only available for simulation.
 #' @return A list object, including the followings
@@ -570,7 +571,7 @@ second.step.detect <- function(data, pts, omega, lambda, mu, alpha_L = 0.25, ver
 #' }
 #' @keywords internal
 #'
-fista.LpS <- function(A, b, lambda, mu, alpha_L = 0.25, 
+fista.LpS <- function(A, b, lambda, mu, alpha_L = 0.25,
                       niter = 100, backtracking = TRUE, x.true){
     tnew = t <- 1
     p <- dim(A)[2]
@@ -579,14 +580,14 @@ fista.LpS <- function(A, b, lambda, mu, alpha_L = 0.25,
     y1 = y2 <- x1
     AtA <- t(A) %*% A
     Atb <- t(A) %*% b
-    
+
     if(backtracking == TRUE){
         L <- norm(A, "F")^2 / 5
         gamma <- 2
     }else{
         L <- norm(A, "F")^2
     }
-    
+
     obj.val = rel.err <- c()
     for(i in 1:niter){
         if(backtracking == TRUE){
@@ -596,7 +597,7 @@ fista.LpS <- function(A, b, lambda, mu, alpha_L = 0.25,
                 y <- y1 + y2
                 prox1 <- prox.sparse.func(y1, y, A, b, 2*L.bar, lambda, AtA, Atb)
                 prox2 <- prox.nuclear.func(y2, y, A, b, 2*L.bar, mu, AtA, Atb)
-                
+
                 ### Restricted solution space
                 for(j in 1:p){
                     for(k in 1:p){
@@ -605,7 +606,7 @@ fista.LpS <- function(A, b, lambda, mu, alpha_L = 0.25,
                         }
                     }
                 }
-                
+
                 prox <- prox1 + prox2
                 if(f.func(prox, A, b) <= Q.func(prox, y, A, b, L.bar, AtA, Atb)){
                     found <- TRUE
@@ -615,7 +616,7 @@ fista.LpS <- function(A, b, lambda, mu, alpha_L = 0.25,
             }
             L <- L.bar
         }
-        x1 <- xnew1 
+        x1 <- xnew1
         x2 <- xnew2
         xnew1 <- prox1
         xnew2 <- prox2
@@ -624,7 +625,7 @@ fista.LpS <- function(A, b, lambda, mu, alpha_L = 0.25,
         y1 <- xnew1 + (t - 1) / tnew * (xnew1 - x1)
         y2 <- xnew2 + (t - 1) / tnew * (xnew2 - x2)
         xnew <- xnew1 + xnew2
-        
+
         obj.val <- c(obj.val, f.func(xnew, A, b) + sparse.pen(xnew1, lambda) + nuclear.pen(xnew2, mu))
         rel.err <- c(rel.err, norm(xnew - x.true, "F") / norm(x.true, "F"))
     }
@@ -639,7 +640,7 @@ fista.LpS <- function(A, b, lambda, mu, alpha_L = 0.25,
 #' @param tau A positive number, threshold
 #' @return A thresholded matrix, or vector
 #' @keywords internal
-#' 
+#'
 shrinkage <- function(y, tau){
     z <- matrix(0, nrow = nrow(y), ncol = ncol(y))
     for(i in 1:nrow(y)){
@@ -656,7 +657,7 @@ shrinkage <- function(y, tau){
 #' @param tau A positive number, threshold
 #' @return A thresholded matrix, or vector
 #' @keywords internal
-#' 
+#'
 shrinkage.lr <- function(y, tau){
     z <- rep(0, length(y))
     for(i in 1:length(y)){
@@ -673,7 +674,7 @@ shrinkage.lr <- function(y, tau){
 #' @param Atb An inner product for design matrix A and corresponding matrix (vector) b
 #' @return Value of gradients
 #' @keywords internal
-#' 
+#'
 gradf.func <- function(x, AtA, Atb){
     return(AtA %*% x - Atb)
 }
@@ -685,7 +686,7 @@ gradf.func <- function(x, AtA, Atb){
 #' @param lambda Tuning parameter
 #' @return Value of nuclear norm penalty term
 #' @keywords internal
-#' 
+#'
 nuclear.pen <- function(x, lambda){
     d <- svd(x)$d
     return(lambda * sum(d))
@@ -693,12 +694,12 @@ nuclear.pen <- function(x, lambda){
 
 
 #' L1-norm penalty for sparse component
-#' @description L1-norm penalty for sparse component 
+#' @description L1-norm penalty for sparse component
 #' @param x Model parameter
 #' @param lambda Tuning parameter
 #' @return Value of l1-norm penalty term
 #' @keywords internal
-#' 
+#'
 sparse.pen <- function(x, lambda){
     return(lambda*sum(x))
 }
@@ -711,7 +712,7 @@ sparse.pen <- function(x, lambda){
 #' @param b Correspond vector or matrix
 #' @return Value of objective function
 #' @keywords internal
-#' 
+#'
 f.func <- function(x, A, b){
     return(0.5 * norm(A %*% x - b, "F")^2)
 }
@@ -728,7 +729,7 @@ f.func <- function(x, A, b){
 #' @param Atb Inner product for design matrix A and correspond vector b
 #' @return Value of function Q
 #' @keywords internal
-#' 
+#'
 Q.func <- function(x, y, A, b, L, AtA, Atb){
     return(f.func(y, A, b) + sum((x - y) * gradf.func(y, AtA, Atb)) + 0.5 * L * norm(x - y, "F")^2)
 }
@@ -746,7 +747,7 @@ Q.func <- function(x, y, A, b, L, AtA, Atb){
 #' @param Atb inner product of design matrix A and correspond vector b
 #' @return Value of proximal function with nuclear norm penalty
 #' @keywords internal
-#' 
+#'
 prox.nuclear.func <- function(w1, y, A, b, L, lambda, AtA, Atb){
     Y <- w1 - (1 / L) * gradf.func(y, AtA, Atb)
     d <- shrinkage.lr(svd(Y)$d, 2*lambda / L)
@@ -766,7 +767,7 @@ prox.nuclear.func <- function(w1, y, A, b, L, lambda, AtA, Atb){
 #' @param Atb inner product of design matrix A and correspond vector b
 #' @return Value of proximal function with l1-norm penalty
 #' @keywords internal
-#' 
+#'
 prox.sparse.func <- function(w1, y, A, b, L, lambda, AtA, Atb){
     Y <- w1 - (1 / L) * gradf.func(y, AtA, Atb)
     return(shrinkage(Y, 2*lambda / L))
@@ -778,13 +779,13 @@ prox.sparse.func <- function(w1, y, A, b, L, lambda, AtA, Atb){
 #' @description objective function, main loss function and penalties
 #' @param x.lr low-rank component
 #' @param x.sparse sparse component
-#' @param A design matrix 
+#' @param A design matrix
 #' @param b correspond vector
 #' @param lambda a tuning parameter for sparse component
 #' @param mu a tuning parameter for low-rank component
 #' @return value of objective function
 #' @keywords internal
-#' 
+#'
 obj.func <- function(x.lr, x.sparse, A, b, lambda, mu){
     ### x.sparse is a list
     m <- length(x.sparse)
@@ -841,7 +842,7 @@ obj.func <- function(x.lr, x.sparse, A, b, lambda, mu){
 #' \describe{
 #'     \item{sizes}{A 2-d numeric vector, indicating the size of time series data}
 #'     \item{true_lag}{True time lags for the process, here is fixed to be 1.}
-#'     \item{true_lagvector}{A vector recording the time lags for different segments, not available under this model setting, 
+#'     \item{true_lagvector}{A vector recording the time lags for different segments, not available under this model setting,
 #'                           here is fixed to be NULL}
 #'     \item{true_cp}{True change points for simulation, a numeric vector}
 #'     \item{true_sparse}{A list of numeric matrices, indicating the true sparse components for all segments}
@@ -866,49 +867,50 @@ obj.func <- function(x.lr, x.sparse, A, b, lambda, mu){
 #' info_ratio <- rep(0.35, 2)
 #' lambda1 = lambda2 = lambda3 <- c(2.5, 2.5)
 #' mu1 = mu2 = mu3 <- c(15, 15)
-#' try_simu <- simu_lstsp(nreps = 3, simu_method = "LS", nob = nob, k = p, 
-#'                        brk = brk, sigma = diag(p), signals = signals, 
-#'                        rank = rank, singular_vals = singular_vals, 
-#'                        info_ratio = info_ratio, sp_pattern = "off-diagonal", 
-#'                        spectral_radius = 0.9, lambda.1 = lambda1, mu.1 = mu1, 
-#'                        lambda.2 = lambda2, mu.2 = mu2, lambda.3 = lambda3, 
+#' try_simu <- simu_lstsp(nreps = 3, simu_method = "LS", nob = nob, k = p,
+#'                        brk = brk, sigma = diag(p), signals = signals,
+#'                        rank = rank, singular_vals = singular_vals,
+#'                        info_ratio = info_ratio, sp_pattern = "off-diagonal",
+#'                        spectral_radius = 0.9, lambda.1 = lambda1, mu.1 = mu1,
+#'                        lambda.2 = lambda2, mu.2 = mu2, lambda.3 = lambda3,
 #'                        mu.3 = mu3, step.size = 5, niter = 20, rolling.skip = 5,
 #'                        cv = FALSE, verbose = TRUE)
 #' summary(try_simu, critical = 5)
 #' }
-simu_lstsp <- function(nreps, simu_method = c("LS"), nob, k, lags = 1, 
-                       lags_vector = NULL, brk, sigma, skip = 50, group_mats = NULL, 
-                       group_type = c("columnwise", "rowwise"), group_index = NULL, 
-                       sparse_mats = NULL, sp_density = NULL, signals = NULL, rank = NULL, 
-                       info_ratio = NULL, sp_pattern = c("off-diagonal", "diagoanl", "random"), 
-                       singular_vals = NULL, spectral_radius = 0.9, alpha_L = 0.25, 
-                       lambda.1 = NULL, mu.1 = NULL, lambda.1.seq = NULL, mu.1.seq = NULL, 
-                       lambda.2, mu.2, lambda.3, mu.3, omega = NULL, h = NULL, 
-                       step.size = NULL, tol = 1e-4, niter = 100, backtracking = TRUE, 
+simu_lstsp <- function(nreps, simu_method = c("LS"), nob, k, lags = 1,
+                       lags_vector = NULL, brk, sigma, skip = 50, group_mats = NULL,
+                       group_type = c("columnwise", "rowwise"), group_index = NULL,
+                       sparse_mats = NULL, sp_density = NULL, signals = NULL, rank = NULL,
+                       info_ratio = NULL, sp_pattern = c("off-diagonal", "diagoanl", "random"),
+                       singular_vals = NULL, spectral_radius = 0.9, alpha_L = 0.25,
+                       lambda.1 = NULL, mu.1 = NULL, lambda.1.seq = NULL, mu.1.seq = NULL,
+                       lambda.2, mu.2, lambda.3, mu.3, omega = NULL, h = NULL,
+                       step.size = NULL, tol = 1e-4, niter = 100, backtracking = TRUE,
                        rolling.skip = 5, cv = FALSE, nfold = NULL, verbose = FALSE){
-    
+
     group_type <- match.arg(group_type)
     sp_pattern <- match.arg(sp_pattern)
-    # storage variables 
+
+    # storage variables
     est_cps <- vector('list', nreps)
     est_sparse_mats <- vector('list', nreps)
     est_lowrank_mats <- vector('list', nreps)
     est_phi_mats <- vector('list', nreps)
     running_times <- rep(0, nreps)
-    
+
     # start fitting
     for(rep in 1:nreps){
         cat(paste("========================== Start replication:", rep, "==========================\n", sep = " "))
-        try <- simu_var(method = simu_method, nob = nob, k = k, lags = lags, lags_vector = lags_vector, 
-                        brk = brk, sigma = sigma, skip = skip, group_mats = group_mats, group_type = group_type, 
-                        group_index = group_index, sparse_mats = sparse_mats, sp_pattern = sp_pattern, 
-                        sp_density = sp_density, signals = signals, rank = rank, info_ratio = info_ratio, 
+        try <- simu_var(method = simu_method, nob = nob, k = k, lags = lags, lags_vector = lags_vector,
+                        brk = brk, sigma = sigma, skip = skip, group_mats = group_mats, group_type = group_type,
+                        group_index = group_index, sparse_mats = sparse_mats, sp_pattern = sp_pattern,
+                        sp_density = sp_density, signals = signals, rank = rank, info_ratio = info_ratio,
                         singular_vals = singular_vals, spectral_radius = spectral_radius, seed = rep)
         data <- as.matrix(try$series)
-        fit <- lstsp(data, lambda.1 = lambda.1, mu.1 = mu.1, lambda.1.seq = lambda.1.seq, 
-                     mu.1.seq = mu.1.seq, lambda.2 = lambda.2, mu.2 = mu.2, 
-                     lambda.3 = lambda.3, mu.3 = mu.3, alpha_L = alpha_L, omega = omega, 
-                     h = h, step.size = step.size, tol = tol, niter = niter, backtracking = backtracking, 
+        fit <- lstsp(data, lambda.1 = lambda.1, mu.1 = mu.1, lambda.1.seq = lambda.1.seq,
+                     mu.1.seq = mu.1.seq, lambda.2 = lambda.2, mu.2 = mu.2,
+                     lambda.3 = lambda.3, mu.3 = mu.3, alpha_L = alpha_L, omega = omega,
+                     h = h, step.size = step.size, tol = tol, niter = niter, backtracking = backtracking,
                      skip = rolling.skip, cv = cv, nfold = nfold, verbose = verbose)
         cat("========================== Finished ==========================\n")
         est_cps[[rep]] <- fit$cp
@@ -916,21 +918,22 @@ simu_lstsp <- function(nreps, simu_method = c("LS"), nob, k, lags = 1,
         est_lowrank_mats[[rep]] <- fit$lowrank_mats
         est_phi_mats[[rep]] <- fit$est_phi
         running_times[rep] <- fit$time
+        cat("\n")
     }
-    
+
     ret <- structure(list(sizes = c(nob, k),
                           true_lag = 1,
-                          true_lagvector = NULL, 
-                          true_cp = brk, 
-                          true_sparse = try$sparse_param, 
+                          true_lagvector = NULL,
+                          true_cp = brk,
+                          true_sparse = try$sparse_param,
                           true_lowrank = try$lowrank_param,
-                          true_phi = try$model_param, 
-                          est_cps = est_cps, 
+                          true_phi = try$model_param,
+                          est_cps = est_cps,
                           est_lag = 1,
-                          est_lagvector = NULL, 
-                          est_sparse_mats = est_sparse_mats, 
-                          est_lowrank_mats = est_lowrank_mats, 
-                          est_phi_mats = est_phi_mats, 
+                          est_lagvector = NULL,
+                          est_sparse_mats = est_sparse_mats,
+                          est_lowrank_mats = est_lowrank_mats,
+                          est_phi_mats = est_phi_mats,
                           running_times = running_times), class = "VARDetect.simu.result")
     return(ret)
 }
